@@ -3,12 +3,12 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
@@ -41,6 +41,13 @@
 /**
  * Implementation of the ConnectionPool interface
  *
+ * This implementation provides a thread-safe, self-managing connection pool.
+ * - Dynamic pool size management between initial and max connections
+ * - Periodic connection reaping to remove idle and non-responsive connections
+ * - Efficient "rolling window" approach: removing old connections from the start
+ *   of the pool vector, adding new ones to the end
+ * - Double-check connection validity: both in reaping and before serving to clients
+ *
  * @file
  */
 
@@ -55,14 +62,14 @@ struct ConnectionPool_S {
         bool doSweep;
         char *error;
         Sem_T alarm;
-	Mutex_T mutex;
-	Vector_T pool;
+        Mutex_T mutex;
+        Vector_T pool;
         Thread_T reaper;
         int sweepInterval;
-	int maxConnections;
+        int maxConnections;
         volatile bool stopped;
         int connectionTimeout;
-	int initialConnections;
+        int initialConnections;
 };
 
 int ZBDEBUG = false;
@@ -80,16 +87,16 @@ void(*AbortHandler)(const char *error) = NULL;
 
 static void _drainPool(T P) {
         while (! Vector_isEmpty(P->pool)) {
-		Connection_T con = Vector_pop(P->pool);
-		Connection_free(&con);
-	}
+                Connection_T con = Vector_pop(P->pool);
+                Connection_free(&con);
+        }
 }
 
 
 static bool _fillPool(T P) {
-	for (int i = 0; i < P->initialConnections; i++) {
+        for (int i = 0; i < P->initialConnections; i++) {
                 Connection_T con = Connection_new(P, &P->error);
-		if (! con) {
+                if (! con) {
                         if (i > 0) {
                                 DEBUG("Failed to fill the pool with initial connections -- %s\n", P->error);
                                 FREE(P->error);
@@ -97,64 +104,116 @@ static bool _fillPool(T P) {
                         }
                         return false;
                 }
-		Vector_push(P->pool, con);
-	}
-	return true;
+                Vector_push(P->pool, con);
+        }
+        return true;
 }
 
 
-static Connection_T _getConnectionWithError(T P, char error[static STRLEN]) {
+static void _mapActive(const void *con, void *ap) {
+        int *active = ap;
+        if (!Connection_isAvailable((Connection_T)con)) *active += 1;
+}
+
+
+static int _active(T P) {
+        int active = 0;
+        Vector_map(P->pool, _mapActive, &active);
+        return active;
+}
+
+
+static inline Connection_T _getAvailableConnection(T P) {
         Connection_T con = NULL;
-        *error = 0;
-        LOCK(P->mutex)
-        {
-                int size = Vector_size(P->pool);
-                for (int i = 0; i < size; i++) {
-                        con = Vector_get(P->pool, i);
-                        if (Connection_isAvailable(con)) {
-                                if (Connection_ping(con)) {
-                                        Connection_setAvailable(con, false);
-                                        goto done;
-                                }
-                        }
-                }
-                con = NULL;
-                if (size < P->maxConnections) {
-                        con = Connection_new(P, &P->error);
-                        if (con) {
-                                Connection_setAvailable(con, false);
-                                Vector_push(P->pool, con);
-                        } else {
-                                snprintf(error, STRLEN - 1, "Failed to create a connection -- %s",
-                                         STR_UNDEF(P->error) ? "unknown error" : P->error);
-                                FREE(P->error);
-                        }
-                } else {
-                        snprintf(error, STRLEN - 1, "Failed to create a connection -- max connections reached");
+        int size = Vector_size(P->pool);
+        for (int i = 0; i < size; i++) {
+                con = Vector_get(P->pool, i);
+                if (Connection_isAvailable(con)) {
+                        Connection_setAvailable(con, false);
+                        return con;
                 }
         }
-done:
-        END_LOCK;
-        if (!con) {
-                DEBUG("%s\n", error);
+        return NULL;
+}
+
+
+static inline Connection_T _createConnection(T P, char error[static STRLEN]) {
+        Connection_T con = Connection_new(P, &P->error);
+        if (con) {
+                LOCK(P->mutex)
+                {
+                        Connection_setAvailable(con, false);
+                        Vector_push(P->pool, con);
+                }
+                END_LOCK;
+        } else {
+                snprintf(error, STRLEN, "Failed to create a connection -- %s",
+                         STR_DEF(P->error) ? P->error : "unknown error");
+                FREE(P->error);
         }
         return con;
 }
 
 
-static int _getActive(T P){
-        int i, n = 0, size = Vector_size(P->pool);
-        for (i = 0; i < size; i++)
-                if (! Connection_isAvailable(Vector_get(P->pool, i))) 
-                        n++; 
-        return n; 
+static Connection_T _getConnection(T P, char error[static STRLEN]) {
+        Connection_T con = NULL;
+        int size = 0;
+        int activeConnections = 0;
+        int availableConnections = 0;
+        *error = 0;
+        LOCK(P->mutex)
+        {
+                size = Vector_size(P->pool);
+                activeConnections = _active(P);
+                availableConnections = size - activeConnections;
+        }
+        END_LOCK;
+        while (availableConnections > 0) {
+                LOCK(P->mutex)
+                {
+                        con = _getAvailableConnection(P);
+                }
+                END_LOCK;
+                if (!con) {
+                        // No more available connections, break to try creation
+                        break;
+                }
+                if (Connection_ping(con)) {
+                        return con;
+                } else {
+                        // Connection failed ping test, remove it and continue
+                        LOCK(P->mutex)
+                        {
+                                Vector_remove(P->pool, Vector_indexOf(P->pool, con));
+                                size = Vector_size(P->pool);
+                                availableConnections--;
+                        }
+                        END_LOCK;
+                        Connection_free(&con);
+                }
+        }
+        // If we're here, either all available connections failed or there were none
+        // Try to create a new connection if the pool isn't full
+        // Note: 'size' might not reflect the current pool size due to concurrent modifications.
+        // We risk potential temporary over-allocation to prioritize getting a creation error
+        // if the database is down. 
+        if (size < P->maxConnections) {
+                con = _createConnection(P, error);
+                if (con) return con;
+        } else {
+                snprintf(error, STRLEN, "Failed to get a connection -- pool is full (max connections reached)");
+        }
+        DEBUG("%s\n", error);
+        return NULL;
 }
 
 
 static int _reapConnections(T P) {
         int n = 0;
-        int x = Vector_size(P->pool) - _getActive(P) - P->initialConnections;
+        int x = Vector_size(P->pool) - _active(P) - P->initialConnections;
         time_t timedout = Time_now() - P->connectionTimeout;
+        // We don't always examine all idle connections in a single run,
+        // but over multiple runs this should cycles through all connections
         for (int i = 0; ((n < x) && (i < Vector_size(P->pool))); i++) {
                 Connection_T con = Vector_get(P->pool, i);
                 if (Connection_isAvailable(con)) {
@@ -164,7 +223,7 @@ static int _reapConnections(T P) {
                                 n++;
                                 i--;
                         }
-                } 
+                }
         }
         return n;
 }
@@ -191,31 +250,33 @@ static void *_doSweep(void *args) {
 
 T ConnectionPool_new(URL_T url) {
         T P;
-	assert(url);
+        assert(url);
         System_init();
-	NEW(P);
+        NEW(P);
         P->url = url;
         Sem_init(P->alarm);
-	Mutex_init(P->mutex);
-	P->maxConnections = SQL_DEFAULT_MAX_CONNECTIONS;
+        Mutex_init(P->mutex);
+        P->doSweep = true;
+        P->sweepInterval = SQL_DEFAULT_SWEEP_INTERVAL;
+        P->maxConnections = SQL_DEFAULT_MAX_CONNECTIONS;
         P->pool = Vector_new(SQL_DEFAULT_MAX_CONNECTIONS);
-	P->initialConnections = SQL_DEFAULT_INIT_CONNECTIONS;
+        P->initialConnections = SQL_DEFAULT_INIT_CONNECTIONS;
         P->connectionTimeout = SQL_DEFAULT_CONNECTION_TIMEOUT;
-	return P;
+        return P;
 }
 
 
 void ConnectionPool_free(T *P) {
         Vector_T pool;
-	assert(P && *P);
+        assert(P && *P);
         pool = (*P)->pool;
         if (! (*P)->stopped)
                 ConnectionPool_stop((*P));
         Vector_free(&pool);
-	Mutex_destroy((*P)->mutex);
+        Mutex_destroy((*P)->mutex);
         Sem_destroy((*P)->alarm);
         FREE((*P)->error);
-	FREE(*P);
+        FREE(*P);
 }
 
 
@@ -228,12 +289,12 @@ URL_T ConnectionPool_getURL(T P) {
 }
 
 
-void ConnectionPool_setInitialConnections(T P, int connections) {
+void ConnectionPool_setInitialConnections(T P, int initialConnections) {
         assert(P);
-        assert(connections >= 0);
+        assert(initialConnections >= 0);
         LOCK(P->mutex)
         {
-                P->initialConnections = connections;
+                P->initialConnections = initialConnections;
         }
         END_LOCK;
 }
@@ -276,41 +337,25 @@ int ConnectionPool_getConnectionTimeout(T P) {
 
 
 void ConnectionPool_setAbortHandler(T P, void(*abortHandler)(const char *error)) {
-        assert(P); 
+        assert(P);
         AbortHandler = abortHandler;
 }
 
 
 void ConnectionPool_setReaper(T P, int sweepInterval) {
         assert(P);
-        assert(sweepInterval>0);
         LOCK(P->mutex)
         {
-                P->doSweep = true;
-                P->sweepInterval = sweepInterval;
+                if (sweepInterval > 0) {
+                        P->doSweep = true;
+                        P->sweepInterval = sweepInterval;
+                } else {
+                        P->doSweep = false;
+                }
         }
         END_LOCK;
 }
 
-
-int ConnectionPool_size(T P) {
-        assert(P);
-        return Vector_size(P->pool);
-}
-
-
-int ConnectionPool_active(T P) {
-        int n = 0;
-        assert(P);
-        LOCK(P->mutex)
-        {
-                n = _getActive(P);
-        }
-        END_LOCK;
-        return n;
-}
-
-
 /* -------------------------------------------------------- Public methods */
 
 
@@ -358,14 +403,14 @@ void ConnectionPool_stop(T P) {
 
 Connection_T ConnectionPool_getConnection(T P) {
         assert(P);
-        return _getConnectionWithError(P, (char[STRLEN]){});
+        return _getConnection(P, (char[STRLEN]){});
 }
 
 
 Connection_T ConnectionPool_getConnectionOrException(T P) {
         assert(P);
         char error[STRLEN] = {};
-        Connection_T con = _getConnectionWithError(P, error);
+        Connection_T con = _getConnection(P, error);
         if (!con) {
                 THROW(SQLException, "%s", error);
         }
@@ -374,21 +419,21 @@ Connection_T ConnectionPool_getConnectionOrException(T P) {
 
 
 void ConnectionPool_returnConnection(T P, Connection_T connection) {
-	assert(P);
+        assert(P);
         assert(connection);
-	if (Connection_isInTransaction(connection)) {
+        if (Connection_inTransaction(connection)) {
                 TRY
                         Connection_rollback(connection);
                 ELSE
                         DEBUG("Failed to rollback transaction -- %s\n", Exception_frame.message);
                 END_TRY;
-	}
-	Connection_clear(connection);
-	LOCK(P->mutex)
-        {
-		Connection_setAvailable(connection, true);
         }
-	END_LOCK;
+        Connection_clear(connection);
+        LOCK(P->mutex)
+        {
+                Connection_setAvailable(connection, true);
+        }
+        END_LOCK;
 }
 
 
@@ -404,6 +449,39 @@ int ConnectionPool_reapConnections(T P) {
 }
 
 
+int ConnectionPool_size(T P) {
+        assert(P);
+        return Vector_size(P->pool);
+}
+
+
+int ConnectionPool_active(T P) {
+        assert(P);
+        int n = 0;
+        LOCK(P->mutex)
+        {
+                Vector_map(P->pool, _mapActive, &n);
+        }
+        END_LOCK;
+        return n;
+}
+
+
+bool ConnectionPool_isFull(T P) {
+        assert(P);
+        bool full = false;
+        LOCK(P->mutex)
+        {
+                full = (_active(P) >= P->maxConnections);
+        }
+        END_LOCK;
+        return full;
+}
+
+
+/* --------------------------------------------------------- Class methods */
+
+
 const char *ConnectionPool_version(void) {
         return ABOUT;
 }
