@@ -26,6 +26,7 @@
 #include "Config.h"
 
 #include <stdio.h>
+#include <string.h>
 
 #include "ResultSet.h"
 #include "PreparedStatement.h"
@@ -84,8 +85,15 @@ void PreparedStatement_free(T *P) {
 
 
 void PreparedStatement_setString(T P, int parameterIndex, const char *x) {
-	assert(P);
-        P->op->setString(P->D, parameterIndex, x);
+        int size = (x) ? (int)strlen(x) : 0;
+        PreparedStatement_setSString(P, parameterIndex, x, size);
+}
+
+
+void PreparedStatement_setSString(T P, int parameterIndex, const char *x, int size) {
+        assert(P);
+        if (size < 0) size = 0;
+        P->op->setString(P->D, parameterIndex, x, size);
 }
 
 
@@ -109,6 +117,7 @@ void PreparedStatement_setDouble(T P, int parameterIndex, double x) {
 
 void PreparedStatement_setBlob(T P, int parameterIndex, const void *x, int size) {
 	assert(P);
+        if (size < 0) size = 0;
         P->op->setBlob(P->D, parameterIndex, x, size);
 }
 
@@ -119,6 +128,12 @@ void PreparedStatement_setTimestamp(T P, int parameterIndex, time_t x) {
 }
 
 
+void PreparedStatement_setNull(T P, int parameterIndex) {
+        assert(P);
+        P->op->setString(P->D, parameterIndex, NULL, 0);
+}
+
+
 /* -------------------------------------------------------- Public methods */
 
 
