@@ -49,7 +49,7 @@ typedef struct param_t {
                 long long llong;
                 MYSQL_TIME timestamp;
         } type;
-        long length;
+        unsigned long length;
 } *param_t;
 #define T PreparedStatementDelegate_T
 struct T {
@@ -106,17 +106,17 @@ static void _free(T *P) {
 }
 
 
-static void _setString(T P, int parameterIndex, const char *x) {
+static void _setString(T P, int parameterIndex, const char *x, int size) {
         assert(P);
         int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->bind[i].buffer_type = MYSQL_TYPE_STRING;
         P->bind[i].buffer = (char*)x;
-        if (! x) {
+        if (size > 0) {
+                P->params[i].length = size;
+                P->bind[i].is_null = 0;
+        } else {
                 P->params[i].length = 0;
                 P->bind[i].is_null = &yes;
-        } else {
-                P->params[i].length = strlen(x);
-                P->bind[i].is_null = 0;
         }
         P->bind[i].length = &P->params[i].length;
 }
@@ -174,12 +174,12 @@ static void _setBlob(T P, int parameterIndex, const void *x, int size) {
         int i = checkAndSetParameterIndex(parameterIndex, P->parameterCount);
         P->bind[i].buffer_type = MYSQL_TYPE_BLOB;
         P->bind[i].buffer = (void*)x;
-        if (! x) {
-                P->params[i].length = 0;
-                P->bind[i].is_null = &yes;
-        } else {
+        if (size > 0) {
                 P->params[i].length = size;
                 P->bind[i].is_null = 0;
+        } else {
+                P->params[i].length = 0;
+                P->bind[i].is_null = &yes;
         }
         P->bind[i].length = &P->params[i].length;
 }
