@@ -6,15 +6,11 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#include "URL.h"
+#include "zdb.h"
+
 #include "Thread.h"
 #include "Vector.h"
-#include "ResultSet.h"
-#include "PreparedStatement.h"
-#include "Connection.h"
-#include "ConnectionPool.h"
 #include "AssertException.h"
-#include "SQLException.h"
 
 
 /**
@@ -96,6 +92,7 @@ static void testPool(const char *testURL) {
                 url = URL_new(testURL);
                 pool = ConnectionPool_new(url);
                 assert(pool);
+                ConnectionPool_setReaper(pool, 0); // disable reaper
                 ConnectionPool_start(pool);
                 ConnectionPool_stop(pool);
                 ConnectionPool_free(&pool);
@@ -130,6 +127,7 @@ static void testPool(const char *testURL) {
                 url = URL_new(testURL);
                 pool = ConnectionPool_new(url);
                 assert(pool);
+                ConnectionPool_setReaper(pool, 0); // disable reaper
                 ConnectionPool_setAbortHandler(pool, TabortHandler);
                 ConnectionPool_start(pool);
                 con = ConnectionPool_getConnection(pool);
@@ -187,11 +185,11 @@ static void testPool(const char *testURL) {
                         PreparedStatement_execute(pre);
                 }
                 /* Add a database null blob value for id = 5 */
-                PreparedStatement_setBlob(pre, 1, NULL, 0);
+                PreparedStatement_setNull(pre, 1);
                 PreparedStatement_setInt(pre, 2, 5);
                 PreparedStatement_execute(pre);
-                /* Add a database null string value for id = 1 */
-                PreparedStatement_setString(pre, 1, NULL);
+                /* Add a database null value for id = 1 */
+                PreparedStatement_setNull(pre, 1);
                 PreparedStatement_setInt(pre, 2, 1);
                 PreparedStatement_execute(pre);
                 /* Add a large blob */
@@ -232,7 +230,7 @@ static void testPool(const char *testURL) {
                         const char *name = ResultSet_getString(rset, 2);
                         double percent = ResultSet_getDoubleByName(rset, "percent");
                         const char *blob = (char*)ResultSet_getBlob(rset, 4, &imagesize);
-                        printf("\t%-5d%-16s%-10.2f%-16.38s\n", id, name ? name : "null", percent, imagesize ? blob : "");
+                        printf("\t%-5d%-16s%-10.2f%-16.38s\n", id, valueOr(name, "null"), percent, valueOr(blob, "null"));
                 }
                 // Column count
                 rset = Connection_executeQuery(con, "select image from zild_t where id=12;");
@@ -336,16 +334,22 @@ static void testPool(const char *testURL) {
                 assert(pool);
                 ConnectionPool_setInitialConnections(pool, 4);
                 ConnectionPool_setMaxConnections(pool, 20);
-                ConnectionPool_setConnectionTimeout(pool, 4);
-                ConnectionPool_setReaper(pool, 4);
+                ConnectionPool_setConnectionTimeout(pool, 2);
+                ConnectionPool_setReaper(pool, 2);
                 ConnectionPool_setAbortHandler(pool, TabortHandler);
                 ConnectionPool_start(pool);
                 assert(4==ConnectionPool_size(pool));
                 printf("Creating 20 Connections..");
-                for (i = 0; i<20; i++)
-                        Vector_push(v, ConnectionPool_getConnection(pool));
+                for (i = 0; i<20; i++) {
+                        Connection_T con = ConnectionPool_getConnection(pool);
+                        assert(con);
+                        Vector_push(v, con);
+                        if (i < 19)
+                                assert(!ConnectionPool_isFull(pool));
+                }
                 assert(ConnectionPool_size(pool) == 20);
                 assert(ConnectionPool_active(pool) == 20);
+                assert(ConnectionPool_isFull(pool));
                 printf("success\n");
                 printf("Closing Connections down to initial..");
                 while (! Vector_isEmpty(v))
@@ -353,14 +357,16 @@ static void testPool(const char *testURL) {
                 assert(ConnectionPool_active(pool) == 0);
                 assert(ConnectionPool_size(pool) == 20);
                 printf("success\n");
-                printf("Please wait 10 sec for reaper to harvest closed connections..");
+                printf("Please wait 5 sec for reaper to harvest closed connections..");
                 Connection_T con = ConnectionPool_getConnection(pool); // Activate one connection to verify the reaper does not close any active
                 fflush(stdout);
-                sleep(10);
+                sleep(5);
                 assert(5 == ConnectionPool_size(pool)); // 4 initial connections + the one active we got above
                 assert(1 == ConnectionPool_active(pool));
+                assert(!ConnectionPool_isFull(pool));
                 printf("success\n");
                 Connection_close(con);
+                assert(0 == ConnectionPool_active(pool));
                 ConnectionPool_stop(pool);
                 ConnectionPool_free(&pool);
                 Vector_free(&v);
@@ -377,6 +383,7 @@ static void testPool(const char *testURL) {
                 url = URL_new(testURL);
                 pool = ConnectionPool_new(url);
                 assert(pool);
+                ConnectionPool_setReaper(pool, 0); // disable reaper
                 ConnectionPool_setAbortHandler(pool, TabortHandler);
                 ConnectionPool_start(pool);
                 con = ConnectionPool_getConnection(pool);
@@ -583,6 +590,7 @@ static void testPool(const char *testURL) {
                         url = URL_new(testURL);
                         pool = ConnectionPool_new(url);
                         assert(pool);
+                        ConnectionPool_setReaper(pool, 0); // disable reaper
                         ConnectionPool_start(pool);
                         Connection_T con = ConnectionPool_getConnection(pool);
                         assert(con);
@@ -592,17 +600,19 @@ static void testPool(const char *testURL) {
                         memset(t, 'x', 4096);
                         t[4095] = 0;
                         for (int i = 0; i < 4; i++) {
-                                PreparedStatement_setBlob(p, 1, t, (i+1)*512); // store successive larger string-blobs to trigger realloc on ResultSet_getBlobByName
+                                // store successive larger string-blobs to trigger realloc on ResultSet_getBlobByName;
+                                // default buffer size is STRLEN and we only realloc more when needed
+                                PreparedStatement_setBlob(p, 1, t, (i+1)*512);
                                 PreparedStatement_setString(p, 2, t);
                                 PreparedStatement_execute(p);
                         }
                         ResultSet_T r = Connection_executeQuery(con, "select image, string from zild_t;");
                         for (int i = 0; ResultSet_next(r); i++) {
                                 ResultSet_getBlobByName(r, "image", &myimagesize);
-                                const char *image = ResultSet_getStringByName(r, "image"); // Blob as image should be terminated
+                                const char *image = ResultSet_getStringByName(r, "image"); // Blob as String is NUL terminated
                                 const char *string = ResultSet_getStringByName(r, "string");
                                 assert(myimagesize == (i+1)*512);
-                                assert(strlen(image) == ((i+1)*512));
+                                assert(strlen(image) == (size_t)((i+1)*512));
                                 assert(strlen(string) == 4095);
                         }
                         p = Connection_prepareStatement(con, "select image, string from zild_t;");
@@ -612,7 +622,7 @@ static void testPool(const char *testURL) {
                                 const char *image = ResultSet_getStringByName(r, "image");
                                 const char *string = (char*)ResultSet_getStringByName(r, "string");
                                 assert(myimagesize == (i+1)*512);
-                                assert(strlen(image) == ((i+1)*512));
+                                assert(strlen(image) == (size_t)((i+1)*512));
                                 assert(strlen(string) == 4095);
                         }
                         Connection_execute(con, "drop table zild_t;");
@@ -629,6 +639,7 @@ static void testPool(const char *testURL) {
                 url = URL_new(testURL);
                 pool = ConnectionPool_new(url);
                 assert(pool);
+                ConnectionPool_setReaper(pool, 0); // disable reaper
                 ConnectionPool_start(pool);
                 Connection_T con = ConnectionPool_getConnection(pool);
                 if (Str_startsWith(testURL, "postgres"))
@@ -648,7 +659,7 @@ static void testPool(const char *testURL) {
                         PreparedStatement_setString(p, 2, "10:12:42");
                 }
                 PreparedStatement_setString(p, 3, "2013-12-28 10:12:42");
-                PreparedStatement_setTimestamp(p, 4, 1387066378);
+                PreparedStatement_setTimestamp(p, 4, 1388225562);
                 PreparedStatement_execute(p);
                 ResultSet_T r = Connection_executeQuery(con, "select * from zild_t");
                 if (ResultSet_next(r)) {
@@ -679,21 +690,21 @@ static void testPool(const char *testURL) {
                         assert(datetime.tm_sec == 42);
                         assert(datetime.TM_GMTOFF == 0);
                         // Check timestamp
-                        assert(timestamp == 1387066378);
+                        assert(timestamp == 1388225562);
                         // Check timestamp as datetime
                         assert(timestampAsTm.tm_year == 2013);
                         assert(timestampAsTm.tm_mon == 11); // Remember month - 1
-                        assert(timestampAsTm.tm_mday == 15);
-                        assert(timestampAsTm.tm_hour == 0);
+                        assert(timestampAsTm.tm_mday == 28);
+                        assert(timestampAsTm.tm_hour == 10);
                         assert(timestampAsTm.tm_min == 12);
-                        assert(timestampAsTm.tm_sec == 58);
+                        assert(timestampAsTm.tm_sec == 42);
                         assert(timestampAsTm.TM_GMTOFF == 0);
                         // Result
-                        printf("\tDate: %s, Time: %s, DateTime: %s\n\tTimestamp as numeric: %ld, Timestamp as string: %s\n",
+                        printf("\tDate: %s, Time: %s, DateTime: %s\n\tTimestamp as numeric: %lld, Timestamp as string: %s\n",
                                ResultSet_getString(r, 1),
                                ResultSet_getString(r, 2),
                                ResultSet_getString(r, 3),
-                               ResultSet_getTimestamp(r, 4),
+                               (long long)ResultSet_getTimestamp(r, 4),
                                ResultSet_getString(r, 4)); // SQLite will show both as numeric
                 }
                 Connection_execute(con, "drop table zild_t;");
