@@ -26,7 +26,7 @@ static void abortHandler(const char *error) {
 }
 
 
-static void testStr() {
+static void testStr(void) {
         printf("============> Start Str Tests\n\n");
         
         printf("=> Test1: copy\n");
@@ -94,7 +94,7 @@ static void testStr() {
         }
         printf("=> Test5: OK\n\n");
         
-        printf("=> Test6: parseInt, parseLong, parseLLong, parseFloat, parseDouble\n");
+        printf("=> Test6: parseInt, parseLong, parseLLong, parseFloat, parseDouble, parseBool\n");
         {
                 char i[STRLEN] = "   -2812 bla";
                 char ll[STRLEN ] = "  2147483642 blabla";
@@ -120,6 +120,23 @@ static void testStr() {
                 }
                 CATCH(SQLException)
                 END_TRY;
+                char t[] = "true";
+                assert(Str_parseBool(t) == true);
+                assert(Str_parseBool("true") == true);
+                assert(Str_parseBool("TRUE") == true);
+                assert(Str_parseBool("yes") == true);
+                assert(Str_parseBool("  Yes  ") == true);
+                assert(Str_parseBool("1") == true);
+                assert(Str_parseBool(" 1 a and 2") == true);
+                assert(Str_parseBool("on") == true);
+                assert(Str_parseBool("enable") == true);
+                assert(Str_parseBool("enabled") == true);
+                assert(Str_parseBool("truelove") == false);
+                assert(Str_parseBool("yesterday") == false);
+                assert(Str_parseBool("1234") == false);
+                assert(Str_parseBool("only") == false);
+                assert(Str_parseBool("enabler") == false);
+                assert(Str_parseBool("enabledment") == false);
         }
         printf("=> Test6: OK\n\n");
         
@@ -128,7 +145,7 @@ static void testStr() {
 }
 
 
-static void testMem() {
+static void testMem(void) {
         printf("============> Start Mem Tests\n\n");
         
         printf("=> Test1: alloc\n");
@@ -163,12 +180,12 @@ static void testMem() {
 }
 
 
-static void testTime() {
+static void testTime(void) {
         printf("============> Start Time Tests\n\n");
                 
         printf("=> Test1: now\n");
         {
-                printf("\tResult: %ld\n", Time_now());
+                printf("\tResult: %lld\n", (long long)Time_now());
         }
         printf("=> Test1: OK\n\n");
 
@@ -351,7 +368,7 @@ static void testTime() {
 }
 
 
-static void testSystem() {
+static void testSystem(void) {
         printf("============> Start System Tests\n\n");
         
         printf("=> Test1: debug\n");
@@ -383,7 +400,7 @@ static void testSystem() {
 }
 
 
-static void testURL() {
+static void testURL(void) {
         URL_T url;
         printf("============> Start URL Tests\n\n");
         
@@ -571,7 +588,7 @@ static void testURL() {
 }
 
 
-static void testVector() {
+static void testVector(void) {
         Vector_T vector;
         printf("============> Start Vector Tests\n\n");
         
@@ -730,13 +747,30 @@ static void testVector() {
                 Vector_free(&vector);
                 assert(vector == NULL);
         }
-        printf("=> Test7: OK\n\n");        
-
+        printf("=> Test7: OK\n\n");
+        
+        printf("=> Test8: indexOf\n");
+        {
+                int i;
+                char b[] = "abcdefghijklmnopqrstuvwxyz";
+                vector = Vector_new(16);
+                assert(vector);
+                for (i = 0; i<26; i++)
+                        Vector_push(vector, &b[i]);
+                assert(Vector_size(vector) == 26);
+                for (i = 0; i < Vector_size(vector); i++) {
+                        assert(i == Vector_indexOf(vector, &b[i]));
+                }
+                Vector_free(&vector);
+                assert(vector == NULL);
+        }
+        printf("=> Test8: OK\n\n");
+        
         printf("============> Vector Tests: OK\n\n");
 }
 
 
-static void testStringBuffer() {
+static void testStringBuffer(void) {
         StringBuffer_T sb;
         printf("============> Start StringBuffer Tests\n\n");
         
